@@ -62,7 +62,8 @@ distutils.filelist.findall = findall    # Fix findall bug in distutils
 from defaultpaths import autostart, autostart_home
 from meta import (author, author_ascii, description, longdesc, domain, name, 
 				  py_maxversion, py_minversion, version, version_tuple, 
-				  wx_minversion, author_email, script2pywname)
+				  wx_minversion, author_email, script2pywname, appstream_id)
+from util_list import intlist
 from util_os import getenvu, relpath, safe_glob
 from util_str import safe_str
 appname = name
@@ -81,7 +82,8 @@ if sys.platform in ("darwin", "win32"):
 		os.environ["PATH"] = safe_str(pth)
 
 config = {"data": ["tests/*.icc"],
-		  "doc": ["LICENSE.txt",
+		  "doc": ["CHANGES.html",
+				  "LICENSE.txt",
 				  "README.html",
 				  "README-fr.html",
 				  "screenshots/*.png",
@@ -104,13 +106,12 @@ config = {"data": ["tests/*.icc"],
 							   "pyglet.input", "pyglet.text",
 							   "pyglet.window", "pyo", "setuptools", "tcl",
 							   "test"],
-					   "darwin": [],
-					   "win32": ["win32com.client.genpy"]},
-		  "package_data": {name: ["argyll_instruments.json",
-								  "beep.wav",
+					   "darwin": ["gdbm"],
+					   "win32": ["gi", "win32com.client.genpy", "yaml"]},
+		  "package_data": {name: ["beep.wav",
 								  "camera_shutter.wav",
 								  "ColorLookupTable.fx",
-								  "lang/*.json",
+								  "lang/*.yaml",
 								  "linear.cal",
 								  "pnp.ids",
 								  "presets/*.icc",
@@ -122,8 +123,6 @@ config = {"data": ["tests/*.icc"],
 								  "report/*.css",
 								  "report/*.html",
 								  "report/*.js",
-								  "technology_strings-1.7.0.json",
-								  "technology_strings-1.7.1.json",
 								  "test.cal",
 								  "theme/*.png",
 								  "theme/*.wav",
@@ -175,7 +174,8 @@ msiversion = ".".join((str(version_tuple[0]),
 plist_dict = {"CFBundleDevelopmentRegion": "English",
 			  "CFBundleExecutable": name,
 			  "CFBundleGetInfoString": version,
-			  "CFBundleIdentifier": ".".join(reversed(domain.split("."))),
+			  "CFBundleIdentifier": ".".join(reversed(domain.split("."))) +
+									"." + name,
 			  "CFBundleInfoDictionaryVersion": "6.0",
 			  "CFBundleLongVersionString": version,
 			  "CFBundleName": name,
@@ -198,6 +198,7 @@ def create_app_symlinks(dist_dir, scripts):
 	# containing the app bundle
 	for src, tgt in [("ref", "Reference"),
 					 ("tests", "Tests"),
+					 ("CHANGES.html", "CHANGES.html"),
 					 ("README.html", "README.html"),
 					 ("README-fr.html", "README-fr.html"),
 					 ("LICENSE.txt", "LICENSE.txt")]:
@@ -210,7 +211,9 @@ def create_app_symlinks(dist_dir, scripts):
 	toolscripts = filter(lambda script: script != name,
 						 [script for script, desc in scripts])
 	for script, desc in scripts:
-		if script in (name, name + "-apply-profiles"):
+		if (script in (name, name + "-apply-profiles",
+					   name + "-eeColor-to-madVR-converter") or
+			script.endswith("-console")):
 			continue
 		toolname = desc.replace(name, "").strip()
 		toolapp = os.path.join(dist_dir, toolname + ".app")
@@ -314,9 +317,9 @@ def get_data(tgt_dir, key, pkgname=None, subkey=None, excludes=None):
 
 def get_scripts(excludes=None):
 	# It is required that each script has an accompanying .desktop file
-	scripts = []
-	desktopfiles = safe_glob(os.path.join(pydir, "..", "misc",
-							 appname.lower() + "*.desktop"))
+	scripts_with_desc = []
+	scripts = safe_glob(os.path.join(pydir, "..", "scripts",
+									 appname.lower() + "*"))
 	def sortbyname(a, b):
 		a, b = [os.path.splitext(v)[0] for v in (a, b)]
 		if a > b:
@@ -325,17 +328,22 @@ def get_scripts(excludes=None):
 			return -1
 		else:
 			return 0
-	desktopfiles.sort(sortbyname)
-	for desktopfile in desktopfiles:
-		if desktopfile.startswith("z-"):
+	scripts.sort(sortbyname)
+	for script in scripts:
+		script = os.path.basename(script)
+		if script == appname.lower() + "-apply-profiles-launcher":
 			continue
-		cfg = ConfigParser()
-		cfg.read(desktopfile)
-		script = cfg.get("Desktop Entry", "Exec").split()[0]
+		desktopfile = os.path.join(pydir, "..", "misc", script + ".desktop")
+		if os.path.isfile(desktopfile):
+			cfg = ConfigParser()
+			cfg.read(desktopfile)
+			script = cfg.get("Desktop Entry", "Exec").split()[0]
+			desc = cfg.get("Desktop Entry", "Name").decode("UTF-8")
+		else:
+			desc = ""
 		if not filter(lambda exclude: fnmatch(script, exclude), excludes or []):
-			scripts.append((script,
-							cfg.get("Desktop Entry", "Name").decode("UTF-8")))
-	return scripts
+			scripts_with_desc.append((script, desc))
+	return scripts_with_desc
 
 
 def setup():
@@ -529,9 +537,15 @@ def setup():
 			if is_rpm_build:
 				doc = os.path.join(os.path.sep, "usr", doc)
 
-	# Use bundled CA file
-	if sdist or do_py2app:
-		config["package_data"][name].append("cacert.pem")
+	# Use CA file from certifi project
+	if do_py2app or do_py2exe:
+		import certifi
+		cacert = certifi.where()
+		if cacert:
+			shutil.copyfile(cacert, os.path.join(pydir, "cacert.pem"))
+			config["package_data"][name].append("cacert.pem")
+		else:
+			print "WARNING: cacert.pem from certifi project not found!"
 
 	# on Mac OS X and Windows, we want data files in the package dir
 	# (package_data will be ignored when using py2exe)
@@ -602,9 +616,9 @@ def setup():
 					print "WARNING: OpenAL32.dll not found!"
 		elif sys.platform != "darwin":
 			# Linux
-			data_files.append((os.path.join(os.path.dirname(data), "appdata"),
+			data_files.append((os.path.join(os.path.dirname(data), "metainfo"),
 							   [os.path.join(pydir, "..", "dist", 
-											 name + ".appdata.xml")]))
+											 appstream_id + ".appdata.xml")]))
 			data_files.append((os.path.join(os.path.dirname(data), 
 											"applications"), 
 							   [os.path.join(pydir, "..", "misc", name.lower() + 
@@ -699,6 +713,11 @@ def setup():
 				data_files.append((os.path.join(os.path.dirname(data), "icons", 
 											 "hicolor", dname, "apps"), 
 								   desktopicons))
+	if do_py2app:
+		data_files.append((os.path.join(data, "scripts"),
+						   [os.path.join(pydir, "..", "scripts",
+										 name.lower() +
+										 "-eecolor-to-madvr-converter")]))
 
 	sources = [os.path.join(name, "RealDisplaySizeMM.c")]
 	if sys.platform == "win32":
@@ -708,31 +727,36 @@ def setup():
 	elif sys.platform == "darwin":
 		macros = [("__APPLE__", None), ("UNIX", None)]
 		libraries = None
-		link_args = ["-framework Carbon", "-framework CoreFoundation", 
-						   "-framework Python", "-framework IOKit"]
-		if not help and ("build" in sys.argv[1:] or 
-						 "build_ext" in sys.argv[1:] or 
-						 (("install" in sys.argv[1:] or 
-						   "install_lib" in sys.argv[1:]) and 
-						  not "--skip-build" in sys.argv[1:])):
-			p = sp.Popen([sys.executable, '-c', '''import os
+		# XXX: Not sure which macOS version exactly removed the need
+		# to specify -framework
+		if intlist(platform.mac_ver()[0].split(".")) >= [10, 7]:
+			link_args = None
+		else:
+			link_args = ["-framework Carbon", "-framework CoreFoundation", 
+							   "-framework Python", "-framework IOKit"]
+			if not help and ("build" in sys.argv[1:] or 
+							 "build_ext" in sys.argv[1:] or 
+							 (("install" in sys.argv[1:] or 
+							   "install_lib" in sys.argv[1:]) and 
+							  not "--skip-build" in sys.argv[1:])):
+				p = sp.Popen([sys.executable, '-c', '''import os
 from distutils.core import setup, Extension
 
 setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r, 
 							 define_macros=%r, extra_link_args=%r)])''' % 
-						  (name, bits, sources, macros, link_args)] + sys.argv[1:], 
-						 stdout = sp.PIPE, stderr = sp.STDOUT)
-			lines = []
-			while True:
-				o = p.stdout.readline()
-				if o == '' and p.poll() != None:
-					break
-				if o[0:4] == 'gcc ':
-					lines.append(o)
-				print o.rstrip()
-			if len(lines):
-				os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.5'
-				sp.call(lines[-1], shell = True)  # fix the library
+							  (name, bits, sources, macros, link_args)] + sys.argv[1:], 
+							 stdout = sp.PIPE, stderr = sp.STDOUT)
+				lines = []
+				while True:
+					o = p.stdout.readline()
+					if o == '' and p.poll() != None:
+						break
+					if o[0:4] == 'gcc ':
+						lines.append(o)
+					print o.rstrip()
+				if len(lines):
+					os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.5'
+					sp.call(lines[-1], shell = True)  # fix the library
 	else:
 		macros = [("UNIX", None)]
 		libraries = ["X11", "Xinerama", "Xrandr", "Xxf86vm"]
@@ -829,7 +853,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 		attrs["exclude_package_data"] = {
 			name: ["RealDisplaySizeMM.c"]
 		}
-		attrs["include_package_data"] = sys.platform in ("darwin", "win32")
+		attrs["include_package_data"] = sys.platform in ("darwin", "win32") and not do_py2app
 		install_requires = [req.replace("(", "").replace(")", "") for req in 
 							requires]
 		attrs["install_requires"] = install_requires
@@ -883,8 +907,12 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 	if do_py2exe:
 		import wx
 		from winmanifest_util import getmanifestxml
+		if platform.architecture()[0] == "64bit":
+			arch = "amd64"
+		else:
+			arch = "x86"
 		manifest_xml = getmanifestxml(os.path.join(pydir, "..", "misc", 
-			name + (".exe.VC90.manifest" if hasattr(sys, "version_info") and 
+			name + (".exe.%s.VC90.manifest" % arch if hasattr(sys, "version_info") and 
 			sys.version_info[:2] >= (2,6) else ".exe.manifest")))
 		tmp_scripts_dir = os.path.join(basedir, "build", "temp.scripts")
 		if not os.path.isdir(tmp_scripts_dir):
@@ -902,25 +930,31 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 			"other_resources": [(24, 1, manifest_xml)],
 			"copyright": u"© %s %s" % (strftime("%Y"), author),
 			"description": desc
-		}) for script, desc in scripts]
+		}) for script, desc in filter(lambda (script, desc):
+									  script != appname.lower() +
+									  "-eecolor-to-madvr-converter" and
+									  not script.endswith("-console"), scripts)]
+
 		# Add profile loader launcher
 		attrs["windows"].append(Target(**{
 			"script": os.path.join(tmp_scripts_dir,
 								   script2pywname(apply_profiles_launcher[0])),
 			"icon_resources": [(1, os.path.join(pydir, "theme", "icons", 
-												appname.lower() + "-apply-profiles" +
+												appname + "-apply-profiles" +
 												".ico"))],
 			"other_resources": [(24, 1, manifest_xml)],
 			"copyright": u"© %s %s" % (strftime("%Y"), author),
 			"description": apply_profiles_launcher[1]
 		}))
-		console_scripts = [name + "-VRML-to-X3D-converter"]
+
+		# Programs that can run with and without GUI
+		console_scripts = [name + "-VRML-to-X3D-converter"]  # No "-console" suffix!
 		for console_script in console_scripts:
 			console_script_path = os.path.join(tmp_scripts_dir,
 											   console_script + "-console")
 			if not os.path.isfile(console_script_path):
 				shutil.copy(os.path.join(basedir, "scripts",
-										 console_script.lower()),
+										 console_script.lower() + "-console"),
 							console_script_path)
 		attrs["console"] = [Target(**{
 			"script": os.path.join(tmp_scripts_dir,
@@ -934,6 +968,19 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 		}) for script, desc in filter(lambda (script, desc):
 									  script2pywname(script) in console_scripts,
 									  scripts)]
+
+		# Programs without GUI
+		attrs["console"].append(Target(**{
+			"script": os.path.join(tmp_scripts_dir,
+								   appname + "-eeColor-to-madVR-converter"),
+			"icon_resources": [(1, os.path.join(pydir, "theme", "icons", 
+												appname + "-3DLUT-maker.ico"))],
+			"other_resources": [(24, 1, manifest_xml)],
+			"copyright": u"© %s %s" % (strftime("%Y"), author),
+			"description": "Convert eeColor 65^3 to madVR 256^3 3D LUT "
+						   "(video levels in, video levels out)"
+		}))
+
 		dist_dir = os.path.join(pydir, "..", "dist", "py2exe.%s-py%s" % 
 								(get_platform(), sys.version[:3]), name + 
 								"-" + version)
@@ -1098,10 +1145,10 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					"theme",
 					"ti1",
 					"x3d-viewer",
+					"CHANGES.html",
 					"LICENSE.txt",
 					"README.html",
 					"README-fr.html",
-					"argyll_instruments.json",
 					"beep.wav",
 					"cacert.pem",
 					"camera_shutter.wav",
@@ -1116,8 +1163,6 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					"pnp.ids",
 					"quirk.json",
 					"linear.cal",
-					"technology_strings-1.7.0.json",
-					"technology_strings-1.7.1.json",
 					"test.cal"
 				]:
 					path = os.path.join(data, fname)
@@ -1127,6 +1172,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				for fname in [
 					"screenshots",
 					"theme",
+					"CHANGES.html",
 					"LICENSE.txt",
 					"README.html",
 					"README-fr.html"
@@ -1141,7 +1187,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 				startmenu_programs = get_special_folder_path("CSIDL_PROGRAMS")
 				for path in (startmenu_programs_common, startmenu_programs):
 					if path:
-						for filename in (name, "LICENSE", "README", 
+						for filename in (name, "CHANGES", "LICENSE", "README", 
 										 "Uninstall"):
 							paths += safe_glob(os.path.join(path, name, 
 															filename + ".lnk"))
@@ -1205,8 +1251,9 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					   "edit"]
 		manifest_in.extend(["include LICENSE.txt", "include MANIFEST", 
 							"include MANIFEST.in", "include README.html", 
-							"include README-fr.html",
-							"include *.pyw", "include use-distutils"])
+							"include README-fr.html", "include CHANGES.html",
+							"include %s*.pyw" % name, "include %s-*.pyw" % name,
+							"include %s-*.py" % name, "include use-distutils"])
 		manifest_in.append("include " + os.path.basename(sys.argv[0]))
 		manifest_in.append("include " + 
 						   os.path.splitext(os.path.basename(sys.argv[0]))[0] + 
@@ -1286,6 +1333,41 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 			return
 		
 		if do_py2app:
+			frameworks_dir = os.path.join(dist_dir, name + ".app",
+										  "Contents", "Frameworks")
+			lib_dynload_dir = os.path.join(dist_dir, name + ".app", "Contents",
+										   "Resources", "lib", "python%s.%s" %
+										   sys.version_info[:2], "lib-dynload")
+			# Fix Pillow (PIL) dylibs not being included
+			pil_dylibs = os.path.join(lib_dynload_dir, "PIL", ".dylibs")
+			if not os.path.isdir(pil_dylibs):
+				import PIL
+				pil_installed_dylibs = os.path.join(os.path.dirname(PIL.__file__),
+													".dylibs")
+				print "Copying", pil_installed_dylibs, "->", pil_dylibs
+				shutil.copytree(pil_installed_dylibs,
+								pil_dylibs)
+				for entry in os.listdir(pil_dylibs):
+					print os.path.join(pil_dylibs, entry)
+				# Remove wrongly included frameworks
+				dylibs_entries = os.listdir(pil_installed_dylibs)
+				for entry in os.listdir(frameworks_dir):
+					if entry in dylibs_entries:
+						dylib = os.path.join(frameworks_dir, entry)
+						print "Removing", dylib
+						os.remove(dylib)
+			import wx
+			if wx.VERSION >= (4, ):
+				# Fix wxPython 4 dylibs being included in wrong location
+				wx_dylibs = os.path.join(lib_dynload_dir, "wx")
+				for entry in os.listdir(frameworks_dir):
+					if entry.startswith("libwx"):
+						dylib = os.path.join(frameworks_dir, entry)
+						lib_dylib = os.path.join(wx_dylibs, entry)
+						print "Moving", dylib, "->", lib_dylib
+						shutil.move(dylib, lib_dylib)
+			
+
 			create_app_symlinks(dist_dir, scripts)
 		
 		if do_py2exe:
