@@ -58,11 +58,13 @@ def IsBW(self):
 	non-equal RGB triplet).
 	
 	"""
+	triplet = set()
 	for i, byte in enumerate(self.GetDataBuffer()):
-			if i % 3 == 2:
-					if byte != prev_byte:
-							return False
-			prev_byte = byte
+		triplet.add(byte)
+		if i % 3 == 2:
+			if len(triplet) != 1:
+				return False
+			triplet = set()
 	return True
 
 wx.Image.IsBW = IsBW
@@ -150,21 +152,43 @@ def SetSaneGeometry(self, x=None, y=None, w=None, h=None):
 	# is completely outside the client area of all displays
 	display_client_rect = self.GetDisplay().ClientArea 
 	if sys.platform not in ("darwin", "win32"): # Linux
-		safety_margin = 40
+		if os.getenv("XDG_SESSION_TYPE") == "wayland":
+			# Client-side decorations
+			safety_margin = 0
+		else:
+			# Assume server-side decorations
+			safety_margin = 40
 	else:
 		safety_margin = 20
 	if not None in (w, h):
 		# Set given size, but resize if needed to fit inside client area
-		min_w, min_h = self.MinSize
-		self.SetSize((max(min(display_client_rect[2], w), min_w), 
-					  max(min(display_client_rect[3] - safety_margin, h),
-						  min_h)))
+		if hasattr(self, "MinClientSize"):
+			min_w, min_h = self.MinClientSize
+		else:
+			min_w, min_h = self.WindowToClientSize(self.MinSize)
+		border_lr = self.Size[0] - self.ClientSize[0]
+		border_tb = self.Size[1] - self.ClientSize[1]
+		self.ClientSize = (min(display_client_rect[2] - border_lr, max(w, min_w)), 
+						   min(display_client_rect[3] - border_tb -
+							   safety_margin, max(h, min_h)))
 	if not None in (x, y):
-		if not display_client_rect.ContainsXY(x, y) or \
-		   not display_client_rect.ContainsRect((x, y, 100, 100)):
-			# If outside client area or near the borders,
-			# move to leftmost / topmost coordinates of client area
-			self.SetPosition(display_client_rect[0:2])
+		if (not display_client_rect.ContainsXY(x, y) or
+			not display_client_rect.ContainsRect((x, y,
+												  self.Size[0], self.Size[1]))):
+			# If outside client area, move into client area
+			xy = [x, y]
+			for i, pos in enumerate([xy,
+									 (x + self.Size[0], y + self.Size[1])]):
+				for j in xrange(2):
+					if (pos[j] > display_client_rect[j] +
+								 display_client_rect[2 + j] or
+						pos[j] < display_client_rect[j]):
+						if i:
+							xy[j] = (display_client_rect[j] +
+									 display_client_rect[2 + j] - self.Size[j])
+						else:
+							xy[j] = display_client_rect[j]
+			self.SetPosition(tuple(xy))
 
 wx.Window.SetSaneGeometry = SetSaneGeometry
 
